#!/usr/bin/perl -w
use Getopt::Long qw(GetOptions);
use IO::Socket::INET;

# timestamp_data
# Program to read raw data stream from a TCP/UDP socket and write it to a file with a timestamp

# Socket reading code inspired by:
# https://www.thegeekstuff.com/2010/07/perl-tcp-udp-socket-programming/

# Set parameters

$version = "2.2";
$build_date = "09/13/2019";

# Set default values for all switches so that non-supplied values don't cause
# "uninitialized value" errors

$host = $verbose = "";
$port = 0;
$protocol = "tcp";
$duration = 300; # 5 minutes

# Parse the command line to get message parameters

GetOptions
  (
  "help" => \$help,
  "duration=s" => \$duration,
  "iphost=s" => \$host,
  "port=s" => \$port,
  "protocol=s" => \$protocol,
  "verbose" => \$verbose,
  )
  or die "Usage: $0 --help for command line assistance\n";

# Check values...

print "\nTimestamp Data (Version $version, Build date: $build_date)\n";

if ($help)
  {
  help ();
  exit;
  }

if ($host eq "")
  {
  print "\nNo host name provided. Program terminating...\n\n";
  exit;
  }

if ($port <= 0 || $port > 65535)
  {
  print "\nInvalid port number $port. Program terminating...\n\n";
  exit;
  }

if (lc($protocol) ne "tcp" && lc($protocol) ne "udp")
  {
  print "\nInvalid protocol $protocol; reset to TCP\n";
  $protocol = "tcp";
  }

if ($duration < 1 || $duration > 604800)
  {
  print "\nInvalid duration value $duration; reset to 300 seconds\n";
  $duration = 300;
  }

# Get current time (Zulu) to create file name

($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime();
$outfile = sprintf ("ais_%04d%02d%02d_%02d%02d%02d.txt", $year+1900, $mon+1, $mday, $hour, $min, $sec);

print "Obtain $duration seconds worth of data from $host:$port/$protocol\n";
print "Output file: $outfile\n\n";

open (OUTFILE, ">", $outfile);

# Creating object interface of IO::Socket::INET modules which internally creates 
# socket, binds and connects to the TCP server running on the specific port

$socket = new IO::Socket::INET (
PeerHost => $host,
PeerPort => $port,
Proto => $protocol,
) or die "ERROR in Socket Creation : $!\n";

print "Connection success to $host:$port/$protocol.\n";

# Set initial timer and count values

$start_time = time();
$num = 0;

# Loop through reading the socket data sent by server until 'duration' seconds elapse
#  from 'start_time'

do
  {
  $current_time = time();
  if (<$socket> ne "")
    {
    $data = <$socket>;
    ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime();
    printf OUTFILE "%04d|%02d|%02d|%02d|%02d|%02d|%d|%s", $year+1900, $mon+1, $mday, $hour, $min, $sec, $current_time, $data;
    if ($verbose) { printf "%04d/%02d/%02d %02d:%02d:%02d %d %s", $year+1900, $mon+1, $mday, $hour, $min, $sec, $current_time, $data; }
    $num++;
    }
  }
  while ($current_time - $start_time < $duration);

print "\n$num records read in ~$duration seconds\n";

$socket->close();
close OUTFILE;

# ********************************************************
# help
# ********************************************************

# Display the help information for the program

sub help
  {
print<< "EOT";

timestamp_data reads a datastream from a TCP/IP host system and writes the data to a
file along with a timestamp. Fields are separated with a "|" character so that the file
can be read as a CSV file.

There are five parameters:

--iphost    A host name or IP address. NO ERROR CHECKING IS PERFORMED ON THIS PARAMETER
--port      TCP/UDP port number. Must be in range 1-65535
--protocol  TCP or UDP [Default = TCP]
--duration  Length of operation, in seconds. Must be in range 1-604800 (one week)
            [Default = 300 (five minutes)]
--verbose   If set, output goes to STDOUT as well as a file

EXAMPLE: timestamp_data.pl --iphost=source.example.com --port=26 --protocol=udp --duration=20
         timestamp_data.pl --iphost=198.51.100.7 --port=8080 --duration=86400 --verbose

Use --help to obtain this help message.

EOT
  return;
  }

